package com.sun.corba.se.impl.activation;

import com.sun.corba.se.spi.activation.ActivatorHelper;
import java.util.Properties;
import org.omg.CORBA.ORB;
import java.lang.reflect.Modifier;
import java.lang.reflect.Method;
import java.util.Date;
import java.io.PrintStream;
import java.io.FileOutputStream;
import java.io.File;

public class ServerMain
{
    public static final int OK = 0;
    public static final int MAIN_CLASS_NOT_FOUND = 1;
    public static final int NO_MAIN_METHOD = 2;
    public static final int APPLICATION_ERROR = 3;
    public static final int UNKNOWN_ERROR = 4;
    public static final int NO_SERVER_ID = 5;
    public static final int REGISTRATION_FAILED = 6;
    private static final boolean debug = false;
    
    public ServerMain() {
    }
    
    public static String printResult(int n) {
        switch (n) {
            case 0: {
                return "Server terminated normally";
            }
            case 1: {
                return "main class not found";
            }
            case 2: {
                return "no main method";
            }
            case 3: {
                return "application error";
            }
            case 5: {
                return "server ID not defined";
            }
            case 6: {
                return "server registration failed";
            }
            default: {
                return "unknown error";
            }
        }
    }
    
    private void redirectIOStreams() {
        try {
            String string = System.getProperty("com.sun.CORBA.activation.DbDir") + System.getProperty("file.separator") + "logs" + System.getProperty("file.separator");
            File file = new File(string);
            String property = System.getProperty("com.sun.CORBA.POA.ORBServerId");
            FileOutputStream out = new FileOutputStream(string + property + ".out", true);
            FileOutputStream out2 = new FileOutputStream(string + property + ".err", true);
            PrintStream out3 = new PrintStream(out, true);
            PrintStream err = new PrintStream(out2, true);
            System.setOut(out3);
            System.setErr(err);
            logInformation("Server started");
        }
        catch (Exception ex) {}
    }
    
    private static void writeLogMessage(PrintStream printStream, String str) {
        printStream.print("[" + new Date().toString() + "] " + str + "\n");
    }
    
    public static void logInformation(String str) {
        writeLogMessage(System.out, "        " + str);
    }
    
    public static void logError(String s) {
        writeLogMessage(System.out, "ERROR:  " + s);
        writeLogMessage(System.err, "ERROR:  " + s);
    }
    
    public static void logTerminal(String str, int status) {
        if (status == 0) {
            writeLogMessage(System.out, "        " + str);
        }
        else {
            writeLogMessage(System.out, "FATAL:  " + printResult(status) + ": " + str);
            writeLogMessage(System.err, "FATAL:  " + printResult(status) + ": " + str);
        }
        System.exit(status);
    }
    
    private Method getMainMethod(Class clazz) {
        Class[] parameterTypes = { String[].class };
        Method declaredMethod = null;
        try {
            declaredMethod = clazz.getDeclaredMethod("main", parameterTypes);
        }
        catch (Exception ex) {
            logTerminal(ex.getMessage(), 2);
        }
        if (!this.isPublicStaticVoid(declaredMethod)) {
            logTerminal("", 2);
            return declaredMethod;
        }
        return declaredMethod;
    }
    
    private boolean isPublicStaticVoid(Method method) {
        int modifiers = method.getModifiers();
        if (!Modifier.isPublic(modifiers) || !Modifier.isStatic(modifiers)) {
            logError(method.getName() + " is not public static");
            return false;
        }
        if (method.getExceptionTypes().length != 0) {
            logError(method.getName() + " declares exceptions");
            return false;
        }
        if (!method.getReturnType().equals(Void.TYPE)) {
            logError(method.getName() + " does not have a void return type");
            return false;
        }
        return true;
    }
    
    private Method getNamedMethod(Class clazz, String name) {
        Class[] parameterTypes = { ORB.class };
        Method declaredMethod;
        try {
            declaredMethod = clazz.getDeclaredMethod(name, parameterTypes);
        }
        catch (Exception ex) {
            return null;
        }
        if (!this.isPublicStaticVoid(declaredMethod)) {
            return null;
        }
        return declaredMethod;
    }
    
    private void run(String[] array) {
        try {
            this.redirectIOStreams();
            String property = System.getProperty("com.sun.CORBA.POA.ORBServerName");
            ClassLoader loader = Thread.currentThread().getContextClassLoader();
            if (loader == null) {
                loader = ClassLoader.getSystemClassLoader();
            }
            Class<Object> clazz;
            try {
                clazz = Class.forName(property);
            }
            catch (ClassNotFoundException ex) {
                clazz = Class.forName(property, true, loader);
            }
            Method mainMethod = this.getMainMethod(clazz);
            if (Boolean.getBoolean("com.sun.CORBA.activation.ORBServerVerify")) {
                if (mainMethod == null) {
                    logTerminal("", 2);
                }
                else {
                    logTerminal("", 0);
                }
            }
            this.registerCallback(clazz);
            mainMethod.invoke(null, new Object[] { array });
        }
        catch (ClassNotFoundException ex2) {
            logTerminal("ClassNotFound exception: " + ex2.getMessage(), 1);
        }
        catch (Exception ex3) {
            logTerminal("Exception: " + ex3.getMessage(), 3);
        }
    }
    
    public static void main(String[] array) {
        new ServerMain().run(array);
    }
    
    private int getServerId() {
        Integer integer = Integer.getInteger("com.sun.CORBA.POA.ORBServerId");
        if (integer == null) {
            logTerminal("", 5);
        }
        return (int)integer;
    }
    
    private void registerCallback(Class clazz) {
        Method namedMethod = this.getNamedMethod(clazz, "install");
        Method namedMethod2 = this.getNamedMethod(clazz, "uninstall");
        Method namedMethod3 = this.getNamedMethod(clazz, "shutdown");
        Properties properties = new Properties();
        properties.put("org.omg.CORBA.ORBClass", "com.sun.corba.se.impl.orb.ORBImpl");
        properties.put("com.sun.CORBA.POA.ORBActivated", "false");
        ORB init = ORB.init(null, properties);
        ServerCallback serverCallback = new ServerCallback(init, namedMethod, namedMethod2, namedMethod3);
        int serverId = this.getServerId();
        try {
            ActivatorHelper.narrow(init.resolve_initial_references("ServerActivator")).active(serverId, serverCallback);
        }
        catch (Exception ex) {
            logTerminal("exception " + ex.getMessage(), 6);
        }
    }
}
